import requests, time, os, sys
import magic


def upload_file(endpoint, datasetId, file, targetName, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    #attempt to detect mimetype
    mime = magic.Magic(mime=True)
    try:
        mimetype = mime.from_file(file)
    except:
        mimetype = None
    #Override mimetype if it's a .csv
    suffix = file.rsplit('.', 1)
    if str(suffix[1]) == 'csv':
        mimetype = 'text/csv'
    #check for keyword args. default folder is '/'. also handle when someone passes in '/'
    description = kwargs.get('description', None)
    folder = kwargs.get('folder', '')
    if folder == '/' or folder.startswith('/'):
        folder=folder.replace('/', '', 1)
    if folder.endswith('/'):
        folder=''.join(folder.rsplit('/', 1))
    folder = '/'+folder
    data = {
    	'description' : description,
    	'mimetype' : mimetype,
        'folder' : folder,
        'name' : targetName
    }
    files =  {
    	"uploadFile": (file, open(str(file), 'rb'), '', {})
    }
    url = endpoint + "/api/pushfile/" + str(datasetId) + '?apikey=' + apikey
    response = requests.post(url, files = files, data = data)
    if response.status_code == 200:
        return response.content
    else:
        sys.exit(str(response.content))

def download_file(endpoint, datasetId, resourceId, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    url = endpoint + "/api/getfile/" + datasetId + "/" + resourceId + '?apikey=' + apikey
    file= requests.get(url)
    if file.status_code == 200:
        return file.content
    else:
        sys.exit(str(file.content))

def delete_resource(endpoint, resourceId, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    url = endpoint + "/api/resource/" + resourceId + '?apikey=' + apikey
    response = requests.delete(url)
    if response.status_code == 200:
        return response.content
    else:
        sys.exit(str(response.content))

def create_table(endpoint, datasetId, schema, tableName, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    url = endpoint + "/api/datasets/" + datasetId + '/tables?apikey=' + apikey
    #check for keyword args. default folder is '/'
    description = kwargs.get('description', None)
    folder = kwargs.get('folder', '')
    folder = '/'+folder
    data = {
        'name' : tableName,
    	'description' : description,
    	'schema' : schema,
        'folder' : folder
    }
    response = requests.post(url, data = data)
    if response.status_code == 200:
        return response.content
    else:
        sys.exit(str(response.content))


def upload_to_table(endpoint, datasetId, file, tableName, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    folder = kwargs.get('folder', '')
    folder = '/'+folder
    if '/' in file:
        name=file.rsplit('/', 1)
        name=str(name[1])
    else:
        name=file
    upload_file(endpoint, datasetId, file, name, description="Appended to "+tableName, folder=folder, apikey=apikey)
    url = endpoint + "/api/datasets/" + datasetId + "/tables/" + tableName + '/loadfile?apikey=' + apikey
    file=folder+name
    #get delimiter if supplied, use comma as default
    delimiter = kwargs.get('delimiter', ',')
    data = {
        'fileName' : file,
        'delimiter' : delimiter,
        'folder' : folder
    }
    time.sleep(3)
    response = requests.post(url, data = data)
    if response.status_code == 200:
        return response.content
    else:
        sys.exit(str(response.content))

def append_to_table(endpoint, datasetId, file, tableName, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    url = endpoint + "/api/datasets/" + datasetId + "/tables/" + tableName + '/loadfile?apikey=' + apikey
    #get delimiter if supplied, use comma as default
    delimiter = kwargs.get('delimiter', ',')
    folder = kwargs.get('folder', '')
    folder = '/'+folder
    data = {
        'fileName' : file,
        'delimiter' : delimiter,
        'folder' : folder
    }
    response = requests.post(url, data = data)
    if response.status_code == 200:
        return response.content
    else:
        sys.exit(str(response.content))

def download_table(endpoint, resourceId, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    url = endpoint + "/api/table/" + resourceId + '?apikey=' + apikey
    file= requests.get(url)
    if file.status_code == 200:
        return file.content
    else:
        sys.exit(str(file.content))

def get_table_metadata(endpoint, datasetId, tablePath, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    if tablePath.startswith('/'):
        tablePath=tablePath.replace('/', '', 1)
    url = endpoint + "/api/datasets/" + datasetId + "/tables/" + tablePath + '?apikey=' + apikey
    metadata = requests.get(url)
    if metadata.status_code == 200:
        return metadata.content
    else:
        sys.exit(str(metadata.content))

def sendDatasetUpdateEmail(endpoint, datasetId):
    #check for apikey in env var or args
    # use a dummy api key for now since auth hasn't been implemented for this yet
    apikey = "QQQQ"
    url = endpoint + "/api/datasetChanged/" + datasetId
    notification = requests.get(url)
    if notification.status_code == 200:
        return notification.content
    else:
        sys.exit(str(notification.content))


def get_query(endpoint, resourceId, *args, **kwargs):
    #check for apikey in env var or args
    apikey = kwargs.get('apikey', None)
    if apikey == None:
        apikey = os.getenv('ENV_PROFILE_APIKEY', None)
    if apikey == None: # better: if item is not None
        sys.exit('ERROR: Please provide your API key either as an environment varible (ENV_PROFILE_APIKEY) or as a keyword argument.')
    url = endpoint + "/api/query/" + resourceId + "?apikey=" + apikey
    query = requests.get(url, headers={"Accept":"text/plain"})
    if query.status_code == 200:
        return query.content
    else:
        sys.exit(str(query.content))
